import json
import datetime

from typing import TypedDict, Any, List

from datacaster.classes import CastDataClass


def _convert_timestamp(timestamp: float) -> str:
    return datetime.datetime.fromtimestamp(timestamp).strftime("%H:%M:%S %d/%m/%Y")


DateString = str


class Tag(TypedDict):
    key: str
    value: Any


class SecretData(CastDataClass):
    arn: str
    created_at: DateString
    name: str
    data: dict
    version_id: str
    __class_config__ = {
        "rename_fields": {
            "ARN": "arn",
            "CreatedDate": "created_at",
            "Name": "name",
            "SecretString": "data",
            "VersionId": "version_id",
        },
        "cast_functions": {"fields": {"data": lambda data: json.loads(data)}, "types": {DateString: _convert_timestamp}},
        "always_cast": ["created_at"],
    }


class Secret(TypedDict):
    """
- Contains the data from an AWS Secrets Manager secret

!!! example

    ```YAML linenums="1"
    arn: arn:aws:secretsmanager:eu-west-2:381957475495:secret:example-secret-yZLBlZ
    created_at: 11:15:35 22/03/2021
    name: example-secret
    data:
      username: example
      password: example
    version_id: 2c6703c0-6ab9-4e8a-9591-5bcbd9469ed0
    ```

!!! info "Secret Keys"

    __arn__: the Amazon resource name

    __created_at__: the date the secret was created

    __name__: the name of the secret

    __data__: a dictionary of secret keys & values

    __version_id__: the ID of the secret version
    """

    __annotations__ = SecretData.__annotations__


class ListedSecretData(CastDataClass):
    arn: str
    created_at: DateString
    last_accessed_at: DateString
    last_changed_at: DateString
    name: str
    tags: List[Tag]
    __class_config__ = {
        "rename_fields": {
            "ARN": "arn",
            "CreatedDate": "created_at",
            "LastAccessedDate": "last_accessed_at",
            "LastChangedDate": "last_changed_at",
            "Name": "name",
            "Tags": "tags",
        },
        "cast_functions": {
            "fields": {"tags": lambda tags: [{"key": tag_dict["Key"], "value": tag_dict["Value"]} for tag_dict in tags]},
            "types": {DateString: _convert_timestamp},
        },
        "always_cast": ["tags", "created_at", "last_accessed_at", "last_changed_at"],
    }


class ListedSecret(TypedDict):
    """
- Contains information about a secret in AWS Secrets Manager

- This dictionary type is outputted when listing all secrets

- No sensitive secret data is contained

!!! example

    ```YAML linenums="1"
    - arn: arn:aws:secretsmanager:eu-west-2:381957475495:secret:example-secret-1-R02UeI
      created_at: '08:40:20 09/03/2021'
      last_accessed_at: 00:00:00 09/03/2021
      last_changed_at: 10:25:18 09/03/2021
      name: example-secret-1
      tags:
      - key: environment
        value: uat
      - key: team
        value: engineering
    ```

!!! info "ListedSecret Keys"

    __arn__: the Amazon resource name

    __created_at__: the date the secret was created

    __last_accessed_at__: the date the secret was last accessed

    __last_changed_at__: the date the secret was last changed

    __name__: the name of the secret

    __tags__: any tags associated with the secret
    """

    __annotations__ = ListedSecretData.__annotations__


SecretList = List[ListedSecret]
