"""Secrets Manager

This module contains actions for reading, creating, & updating secrets in AWS Secrets Manager.

!!! danger "PPA Appliance & Vault Requirements"

    This plugin requires PPA version 2.7.1 or later.

    You must have an AWS Secrets Manager provisioned as a _Vault_ inside PPA to use this plugin.
"""
from typing import Optional, Dict, Any

from ppa import ui

from ._secrets_manager import _read_secret, _list_secrets, _create_secret, _update_secret, _delete_secret
from .records import Secret, SecretList


def read(secret: str, secrets_manager: Optional[str] = None) -> Secret:
    """Read a secret.

!!! question "Minimum Plugin Version: 2.0.0"

??? info "Input"

    - __secret:__ the name of the secret

    - __secrets_manager:__ the name of the AWS Secrets Manager (required if multiple are configured)

??? success "Output"

    A single [Secret](../inputs_outputs/#secret).

??? example

    ```YAML linenums="1"
    - aws_secrets_manager.secrets.read:
        secret: ansible_runner_key
      save: secret
    ```
"""
    return _read_secret(secret=secret, secrets_manager=secrets_manager)


def list(secrets_manager: Optional[str] = None) -> SecretList:
    """List all secrets.

This action only shows which secrets exist, it does not reveal any sensitive secret data.

!!! question "Minimum Plugin Version: 2.0.0"

??? info "Input"

    - __secrets_manager:__ the name of the AWS Secrets Manager (required if multiple are configured)

??? success "Output"

    A list of [ListedSecret](../inputs_outputs/#listedsecret) dictionaries.

??? example

    ```YAML linenums="1"
    - aws_secrets_manager.secrets.list:
      save: secret_list
    ```
"""
    return _list_secrets(secrets_manager=secrets_manager)


def create(
    name: str,
    data: Dict[str, Any],
    tags: Optional[Dict[str, Any]] = None,
    description: Optional[str] = None,
    secrets_manager: Optional[str] = None,
) -> None:
    """Create a new secret.

!!! question "Minimum Plugin Version: 2.0.0"

??? info "Input"

    - __name:__ the name of the secret

    - __data:__ a dictionary containing the secret keys & values

    - __tags:__ optional dictionary containing tags for the secret (defaults to empty)

    - __description:__ optional description for the secret (defaults to empty)

    - __secrets_manager:__ the name of the AWS Secrets Manager (required if multiple are configured)

??? success "Output"

    Nothing is outputted by this action.

??? example

    {% raw %}
    ```YAML linenums="1"
    - aws_secrets_manager.secrets.create:
        name: slack-webhook
        data:
          slack_webhook_url: "{{ webhook_url }}"
        tags:
          team: engineering
          environment: production
        description: Engineering Slack channel incoming webhook url
    ```
    {% endraw %}
"""
    return _create_secret(name=name, description=description, data=data, tags=tags, secrets_manager=secrets_manager)


def update(secret: str, data: Dict[str, Any], secrets_manager: Optional[str] = None) -> None:
    """Create a new version of the secret with the supplied data.

!!! question "Minimum Plugin Version: 2.0.0"

??? info "Input"

    - __name:__ the secret to update

    - __data:__ a dictionary containing the secret keys & values

    - __secrets_manager:__ the name of the AWS Secrets Manager (required if multiple are configured)

!!! warning "Updating Secrets"

    This action creates a new secret version containing **only** the keys & values supplied in `data`.

    Any keys that exist in the current secret **will not** exist in the new version unless a value is supplied.

??? success "Output"

    Nothing is outputted by this action.

??? example

    {% raw %}
    ```YAML linenums="1"
    - aws_secrets_manager.secrets.update:
        secret: slack-webhook
        data:
          slack_webhook_url: "{{ webhook_url }}"
    ```
    {% endraw %}
"""
    return _update_secret(secret=secret, data=data, secrets_manager=secrets_manager)


def delete(
    secret: str, force: Optional[bool] = False, recovery_days: Optional[int] = None, secrets_manager: Optional[str] = None
) -> None:
    """Delete a secret.

!!! question "Minimum Plugin Version: 2.0.0"

??? info "Input"

    - __secret:__ the secret to delete

    - __force:__ set to `true` to force delete the secret with **no** recovery (defaults to `false`)

    - __recovery_days:__ optional number of days to set as the recovery period (defaults to `30`)

    - __secrets_manager:__ the name of the AWS Secrets Manager (required if multiple are configured)

!!! warning "Force Deletion & Recovery"

    You cannot supply both the **force** & **recovery_days** inputs.

??? success "Output"

    Nothing is outputted by this action.

??? example

    ```YAML linenums="1"
    - aws_secrets_manager.secrets.delete:
        secret: slack-webhook
        recovery_days: 14
    ```
"""
    if all([force, recovery_days]):
        message = "The delete action cannot accept both the 'force' & 'recovery_days' inputs."
        ui.output_error(message)
        raise ValueError(message)
    return _delete_secret(secret=secret, force=force, recovery_days=recovery_days, secrets_manager=secrets_manager)
